/*
 *  Petal.java
 *  Copyright Simon Hern 2008
 *  Contact: dishmoth@yahoo.co.uk, www.dishmoth.com
 */

package centipede;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;

// a decorative bit of a Flower
public class Petal {

  // shape
  static private final double kMajorAxis = 15.0,
                              kMinorAxis = 5.0,
                              kOffset    = 5.0;
  static private Ellipse2D kShape = null;
  
  // drawing details
  static private final Color kLineColour = new Color(0.0f, 0.0f, 0.0f);
  static private final BasicStroke kLineStroke = new BasicStroke(0.5f);

  // how the petal detaches
  static private final double kDetachSpeed = 1.0,
                              kMaxAngVel   = 0.1,
                              kGravity     = 0.2;
  
  // position (centre of petal)
  private double mXPos,
                 mYPos,
                 mXVel,
                 mYVel,
                 mYAcc;
  
  // angle (relative to centre)
  private double mAngle,
                 mAngVel;
  
  // fill colour 
  private Color mColour;
  
  // prepare
  static public void initialize() {
    
    if ( kShape != null ) return;
    
    kShape = new Ellipse2D.Double(-kMajorAxis, -kMinorAxis,
                                  2*kMajorAxis, 2*kMinorAxis);

  } // initialize()
  
  // constructor (
  public Petal(int xCentre, int yCentre, double theta, Color colour) {
    
    initialize();
    
    mXPos = xCentre + (kMajorAxis+kOffset)*Math.cos(theta);
    mYPos = yCentre + (kMajorAxis+kOffset)*Math.sin(theta);
    
    mXVel = mYVel = mYAcc = 0.0;
    
    mAngle = theta;
    mAngVel = 0.0;
    
    mColour = colour;
    
  } // constructor
  
  // make the petal fall
  public void detach() {
    
    assert( mXVel == 0.0 && mYVel == 0.0 && mAngVel == 0.0 );
    
    mXVel = kDetachSpeed*Math.cos(mAngle);
    mYVel = kDetachSpeed*Math.sin(mAngle);
 
    mAngVel = kMaxAngVel * Env.randomDouble(-1.0,+1.0);
 
    mYAcc = kGravity;
    
  } // detach()
  
  // move and animate
  public void advance(int speed) {
    
    mXPos += mXVel + speed;
    mYPos += mYVel;
    mYVel += mYAcc;
    mAngle += mAngVel;
    
  } // advance
  
  // display
  public void draw(Graphics2D g2) {
    
    if ( mYPos > Env.screenHeight()+kMajorAxis ) return;
    
    AffineTransform at = AffineTransform.getTranslateInstance(mXPos, mYPos);
    at.rotate(mAngle);
    Shape s = at.createTransformedShape(kShape);
    
    g2.setColor(mColour);
    g2.fill(s);
    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
                        RenderingHints.VALUE_ANTIALIAS_ON);
    g2.setColor(kLineColour);
    g2.setStroke(kLineStroke);
    g2.draw(s);
    
  } // draw()
  
} // class Petal
