/*
 *  Picket.java
 *  Copyright Simon Hern 2008
 *  Contact: dishmoth@yahoo.co.uk, www.dishmoth.com
 */

package centipede;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;

// a bit of a fence
public class Picket {

  // sizes of stuff
  static private final double kRelativeHeight = 3.0;
  static private final double kBarYPos1       = 0.35,
                              kBarYPos2       = 0.7,
                              kBarHeight      = 0.15;
  static private final double kPostWidth      = 0.7,
                              kPostTop        = 0.15;
  
  // colours of stuff
  static private final Color kBlankColour = new Color(0, 0, 0, 0),
                             kFenceColour = new Color(1.0f, 1.0f, 1.0f),
                             kLineColour  = new Color(0.0f, 0.0f, 0.0f);
  static private final BasicStroke kLineStroke = new BasicStroke(0.5f);
  
  // the image
  private BufferedImage mImage = null;
  
  // constructor
  public Picket(int width) {
    
    int height = (int)Math.round( kRelativeHeight * width );
    
    mImage = Env.createTranslucentImage(width, height);
    Graphics2D g2 = mImage.createGraphics();
    g2.setBackground(new Color(0,0,0,0));
    g2.clearRect(0, 0, width, height);

    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
                        RenderingHints.VALUE_ANTIALIAS_ON);

    int barY1 = (int)Math.round(height * kBarYPos1),
        barY2 = (int)Math.round(height * kBarYPos2),
        barH  = (int)Math.round(height * kBarHeight);
    
    g2.setColor(kFenceColour);
    g2.fillRect(0, barY1, width, barH);
    g2.fillRect(0, barY2, width, barH);
  
    g2.setColor(kLineColour);
    g2.setStroke(kLineStroke);
    g2.drawLine(0, barY1, width, barY1);
    g2.drawLine(0, barY1+barH, width, barY1+barH);
    g2.drawLine(0, barY2, width, barY2);
    g2.drawLine(0, barY2+barH, width, barY2+barH);
    
    double postX1 = 0.5*width,
           postX2 = (0.5 + 0.5*kPostWidth)*width,
           postX3 = (0.5 - 0.5*kPostWidth)*width,
           postY1 = 1.0,
           postY2 = kPostTop*height,
           postY3 = height - 1.0;
    GeneralPath post = new GeneralPath(); // warning: JDK1.5
    post.moveTo((float)postX1, (float)postY1);
    post.lineTo((float)postX2, (float)postY2);
    post.lineTo((float)postX2, (float)postY3);
    post.lineTo((float)postX3, (float)postY3);
    post.lineTo((float)postX3, (float)postY2);
    post.closePath();
    
    g2.setColor(kFenceColour);
    g2.fill(post);
  
    g2.setColor(kLineColour);
    g2.setStroke(kLineStroke);
    g2.draw(post);
    
    g2.dispose();
    
  } // constructor
  
  // accessors
  public int width() { return mImage.getWidth(); }
  public int height() { return mImage.getHeight(); }
  
  // display at the specified position
  public void draw(Graphics2D g2, int x, int y) {
    
    g2.drawImage(mImage, x, y, null);
    
  } // draw()
  
} // class Picket
